
from django.urls import path, include
from django.conf.urls.i18n import i18n_patterns

from tecdoc import views


app_name = 'tecdoc'


urlpatterns = [
    path('search/', views.find_article, name='search')
]

app_urls = i18n_patterns(
    path('tecdoc/', include((urlpatterns, app_name)))
)





class TecDocRouter(object):

    def db_for_read(self, model, **hints):

        if model._meta.app_label == 'tecdoc':
            return 'tecdoc'

        return None

    def db_for_write(self, model, **hints):

        if model._meta.app_label == 'tecdoc':
            return 'tecdoc'

        return None

    def allow_relation(self, obj1, obj2, **hints):

        if obj1._meta.app_label == 'tecdoc' or \
                obj2._meta.app_label == 'tecdoc':
            return True

        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):

        if app_label == 'tecdoc':
            return db == 'tecdoc'

        return None




import re

from django.db.models.functions import Lower

from tecdoc.models import *
from tecdoc.constants import ADDITIONAL_CODES_MAX_LENGTH, CYRILLIC_MAP


def clean_code(text):

    result = re.sub(r'[\W_]+', '', text).lower()

    for k, v in CYRILLIC_MAP.items():
        result = result.replace(k, v)

    return result


def get_supplier(name):
    return Supplier.objects.filter(matchcode=name.upper()).first()


def get_crosses_text(supplier, code, initial=None, clean=False):

    if not supplier or not code:
        return ''

    text = initial or ''

    new_codes = Cross.objects.filter(
        article_number=code.upper(),
        supplier=supplier
    ).values_list('oen_br', flat=True)

    new_codes = set(new_codes)

    new_codes |= set(text.replace('\r', '').split('\n'))

    new_codes = filter(bool, new_codes)

    if clean:
        new_codes = map(clean_code, new_codes)

    return '\n'.join(new_codes)


def clean_additional_codes(supplier_name, code, initial=None):

    supplier = get_supplier(supplier_name)

    return get_crosses_text(
        supplier,
        code,
        initial,
        clean=True
    )[:ADDITIONAL_CODES_MAX_LENGTH]




from django.db.models import Q

from django.utils.translation import ugettext_lazy as _

from cap.filters import InputFilter

from tecdoc.utils import clean_code


class ProductCodeFilter(InputFilter):

    parameter_name = 'product_code'
    title = _('Product code')

    def queryset(self, request, queryset):

        code = self.value()

        if not code:
            return

        cleaned_code = clean_code(code)

        return queryset.filter(
            Q(clean_code=cleaned_code) |
            Q(additional_codes__icontains=cleaned_code)
        )




from django.shortcuts import render
from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required


@staff_member_required
def find_article(request):

    code = request.GET.get('code', '')

    articles = []

    return render(request, 'tecdoc/search.html', {
        'articles': articles,
        'code': code,
        **admin.site.each_context(request)
    })




from django.db import models
from django.utils.translation import ugettext_lazy as _

from tecdoc.constants import ADDITIONAL_CODES_MAX_LENGTH


class Manufacturer(models.Model):

    description = models.CharField(max_length=255)

    def __str__(self):
        return self.description

    class Meta:
        db_table = 'manufacturers'


class Supplier(models.Model):

    description = models.CharField(max_length=255)

    matchcode = models.CharField(max_length=255)

    def __str__(self):
        return self.description

    class Meta:
        db_table = 'suppliers'


class ArticleAttribute(models.Model):

    supplier = models.ForeignKey(
        Supplier,
        on_delete=models.PROTECT,
        db_column='supplierid')

    article_number = models.CharField(
        max_length=255,
        db_column='datasupplierarticlenumber',
        primary_key=True)

    description = models.TextField(
        db_column='description')

    value = models.TextField(
        db_column='displayvalue')

    def __str__(self):
        return self.article_number

    class Meta:
        db_table = 'article_attributes'


class ArticleEAN(models.Model):

    supplier = models.ForeignKey(
        Supplier,
        on_delete=models.PROTECT,
        db_column='supplierid')

    article_number = models.CharField(
        max_length=255,
        db_column='datasupplierarticlenumber',
        primary_key=True)

    ean = models.CharField(max_length=255)

    def __str__(self):
        return self.ean

    class Meta:
        db_table = 'article_ean'


class Cross(models.Model):

    manufacturer = models.ForeignKey(
        Manufacturer,
        on_delete=models.PROTECT,
        db_column='manufacturerId')

    oen_br = models.CharField(
        max_length=255,
        db_column='OENbr')

    supplier = models.ForeignKey(
        Supplier,
        on_delete=models.PROTECT,
        db_column='SupplierId')

    article_number = models.CharField(
        max_length=255,
        db_column='PartsDataSupplierArticleNumber',
        primary_key=True)

    def __str__(self):
        return self.article_number

    class Meta:
        db_table = 'article_cross'


class Article(models.Model):

    supplier = models.ForeignKey(
        Supplier,
        on_delete=models.PROTECT,
        db_column='supplierId')

    article_number = models.CharField(
        max_length=255,
        db_column='DataSupplierArticleNumber',
        primary_key=True)

    clean_article_number = models.CharField(
        max_length=255,
        db_column='FoundString')

    description = models.TextField(
        db_column='NormalizedDescription')

    def get_attributes(self):
        return ArticleAttribute.objects.filter(
            article_number=self.article_number,
            supplier_id=self.supplier_id)

    def get_crosses(self):
        return ArticleAttribute.objects.filter(
            article_number=self.article_number,
            supplier_id=self.supplier_id)

    def get_ean_codes(self):
        return ArticleEAN.objects.filter(article_number=self.article_number)

    def get_images(self):
        return Image.objects \
         .filter(
            article_number=self.article_number,
            supplier_id=self.supplier_id)

    def __str__(self):
        return self.article_number

    class Meta:
        db_table = 'articles'


class ArticleOE(models.Model):

    supplier = models.ForeignKey(
        Supplier,
        on_delete=models.PROTECT,
        db_column='supplierId')

    article_number = models.CharField(
        max_length=255,
        db_column='DataSupplierArticleNumber',
        primary_key=True)

    manufacturer = models.ForeignKey(
        Manufacturer,
        on_delete=models.PROTECT,
        db_column='manufacturerId')

    oen_br = models.CharField(
        max_length=255,
        db_column='OENbr')

    def __str__(self):
        return self.article_number

    class Meta:
        db_table = 'article_oe'


class Image(models.Model):

    supplier = models.ForeignKey(
        Supplier,
        on_delete=models.PROTECT,
        db_column='supplierId')

    article_number = models.CharField(
        max_length=255,
        db_column='DataSupplierArticleNumber')

    picture_name = models.CharField(
        max_length=255,
        db_column='PictureName',
        primary_key=True)

    def __str__(self):
        return self.picture_name

    class Meta:
        db_table = 'article_images'


class AdditionalCodesField(models.TextField):

    def __init__(
            self,
            verbose_name=_('Additional codes'),
            blank=True,
            db_index=True,
            max_length=ADDITIONAL_CODES_MAX_LENGTH,
            *args,
            **kwargs):

        super().__init__(
            verbose_name=verbose_name,
            blank=blank,
            db_index=db_index,
            max_length=max_length,
            *args,
            **kwargs)




def setup_settings(settings, is_prod, **kwargs):

    if 'TECDOC_HOST' in settings:
        settings['DATABASES'] = {
            'tecdoc': {
                'ENGINE': 'django.db.backends.mysql',
                'NAME': 'tecdoc',
                'USER': 'tecdoc',
                'HOST': settings['TECDOC_HOST'],
                'PASSWORD': settings['TECDOC_PASSWORD']
            },
            **settings.get('DATABASES', {})
        }

    settings['DATABASE_ROUTERS'] = settings.get('DATABASE_ROUTERS', []) + [
        'tecdoc.routers.TecDocRouter'
    ]
